
# Copyright (C) Johan Ceuppens 2010
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.


import pygame
from pygame.locals import *
from stateimagelibrary import *
from playerbaseclass import *
from playerbaserace import *
from playerfighter import *
from playermagicuser import *
from playerthief import *
from playerkatta import *
from playerelf import *
from playerhuman import *
from playergnoll import *
from playerdrow import *
from selectorrace import *
from selectorclass import *
from rng import *
from strength import *
from dexterity import *
from intelligence import *
from constitution import *
from wisdom import *
from charisma import *
from armorclass import *
from agility import *
from vitality import *
from luck import *
from weaponuse import *
from parry import *
from dodge import *
from stealth import *
from picklocks import *
from throwing import *
from climbing import *
from communicate import *
from honor import *
from stamina import *
from mana import *
from magic import *
from health import *

class PlayerBase(PlayerBaseRace,PlayerBaseClass):
    "Player"
    def __init__(self,PLAYERRACE,PLAYERCLASS,heartmeter):
       	classByType = {
                PlayerBaseClass0.FIGHTER : PlayerFighter,
                PlayerBaseClass0.MAGICUSER : PlayerMagicUser,
                PlayerBaseClass0.THIEF : PlayerThief,
        }
	raceByType = {
                PlayerBaseRace0.HUMAN : PlayerHuman,
                PlayerBaseRace0.KATTA : PlayerKatta,
                PlayerBaseRace0.ELF : PlayerElf,
                PlayerBaseRace0.DROW : PlayerDrow,
                PlayerBaseRace0.GNOLL : PlayerGnoll,
       }

###        PLAYERRACE().__init__(self,heartmeter)
###        PLAYERCLASS().__init__(self,heartmeter)

        self.race = raceByType[PLAYERRACE](heartmeter)
        self.klass = classByType[PLAYERCLASS](heartmeter)

        self.hitpoints = 100
        self.manapoints = 30
	self.priestpoints = 0
	# The type of enemy defeated lets this flcutuate between -x and x
	self.deityenemydefeatedpoints = 0
	# This is the points for going evil (negative), neutral (~0) and good (positive) 
        self.globaldeitypoints = 0
	# These are the points for gaining priest powers (in the priestspells)
        self.deitypoints = 30

        self.maxhitpoints = self.hitpoints
        self.maxmanapoints = self.manapoints
        self.maxdeitypoints = self.deitypoints 

	self.strength = Strength(10)
	self.dexterity = Dexterity(10)
	self.intelligence = Intelligence(12)
	self.constitution = Constitution(self.maxhitpoints)
	self.wisdom = Wisdom(12)
	self.charisma = Charisma(12)
	self.armorclass = ArmorClass(10)

	self.agility = Agility(120)
	self.vitality = Vitality(120)
	self.luck = Luck(120)
	self.weaponuse = WeaponUse(120)
	self.parry = Parry(100)
	self.dodge = Dodge(100)
	self.stealth = Stealth(110)
	self.picklocks = PickLocks(70)
	self.throwing = Throwing(100)
	self.climbing = Climbing(90)
	self.communicate = Communicate(110)
	self.honor = Honor(100)
	self.stamina = Stamina(self.maxhitpoints)
	self.mana = Mana(self.maxmanapoints)
	self.health = Health(self.maxhitpoints)

	self.rng = RNG(self)        

	# set str according to class
	self.setraceclassstrength()

	# set dex according to class
	self.setraceclassdexterity()

	# set hp according to class
	self.setclasshitpoints()

	# set mana according to class
	self.setclassmanapoints()

	# set deity watching according to class
	self.setclassdeitypoints()

	self.heartmeter = heartmeter
        self.direction = "right"

        self.x = 280 
        self.y = 300 
        self.w = 72
        self.h = 72

        self.fightcounter = 0

    def drawstatic(self, screen):
        # NOTE
        if self.fightcounter > 0:
            self.fightcounter += 1
            if self.fightcounter > 10:
                self.fightcounter = 0
            self.stimlibfight.draw(screen,self.x,self.y)
            return
        if self.direction == "right":
            self.stimlibright.drawstatic(screen,self.x,self.y,0)
        if self.direction == "left":
            self.stimlibleft.drawstatic(screen,self.x,self.y,0)

    def drawclimbing(self, screen):
	self.stimlibclimbing.draw(screen,self.x,self.y)

    def drawduck(self, screen):
	self.stimlibduck.draw(screen,self.x,self.y)
	
    def draw(self, screen):
        # NOTE
        if self.fightcounter > 0:
            self.fightcounter += 1
            if self.fightcounter > 10:
                self.fightcounter = 0
            self.stimlibfight.draw(screen,self.x,self.y)#FIXME
            return
        
        if self.direction == "right":
            self.stimlibright.draw(screen,self.x,self.y)
        if self.direction == "left":
            self.stimlibleft.draw(screen,self.x,self.y)

    def fight(self,room):
        self.fightcounter = 1
##        self.x -= 30
##        self.y -= 30
##        self.w += 30
##        self.h += 30
        
        o = room.collidesword(room,self)
        
        if o:
            print 'hit!'
            room.hitwithsword(o)
##        self.x += 30
##        self.y += 30
##        self.w -= 30
##        self.h -= 30
##

        
    def hit(self):
	self.heartmeter.index -= 1 
	if self.heartmeter.index <= 0:
		return 0 #FIXME1 FIX for gameover when collision with enemies 
	else:
		return 0	

    def askpicture(self):
        return './pics/taskbar-PC.bmp'
	
    def askrace(self):
	return self.race.askrace()

    def askclass(self):
	return self.klass.askclass()

    def setraceclassstrength(self):

	strength = self.rng.rolld10() + 10

	if self.askclass() == "Magic User":
		strength -= 1
	elif self.askclass() == "Rogue":
		strength += 1 
	else: # Fighter
		strength += 3
	if self.askrace() == "Gnoll":
		strength += 2	
	elif self.askrace() == "Human":
		strength += 1	
	self.strength = Strength(strength)

    def setraceclassdexterity(self):

	dexterity = self.rng.rolld10() + 10

	if self.askclass() == "Magic User":
		dexterity -= 1
	elif self.askclass() == "Rogue":
		dexterity += 3
	else: # Fighter
		dexterity += 2
	if self.askrace() == "Elf" or self.askrace() == "Katta":
		dexterity += 1	
	self.dexterity = Dexterity(dexterity)

    def setclasshitpoints(self):
	if self.askclass() == "Magic User":
		self.hitpoints = 70
		self.maxhitpoints = 70
	elif self.askclass() == "Rogue":
		self.hitpoints = 90
		self.maxhitpoints = 90
	else: # Fighter
		self.hitpoints = 100
		self.maxhitpoints = 100	

    def setclassmanapoints(self):
	if self.askclass() == "Magic User":
		self.manapoints = 70
		self.maxmanapoints = 70
	elif self.askclass() == "Rogue":
		self.manapoints = 40
		self.maxmanapoints = 40
	else: # Fighter
		self.manapoints = 30
		self.maxmanapoints = 30	

    def setclassdeitypoints(self):
	if self.askclass() == "Magic User":
		self.globaldeitypoints = 0
		self.deitypoints = 40
		self.maxdeitypoints = 40
	elif self.askclass() == "Rogue":
		self.globaldeitypoints = 0
		self.deitypoints = 15
		self.maxdeitypoints = 15
	else: # Fighter
		self.globaldeitypoints = 0
		self.deitypoints = 30
		self.maxdeitypoints = 30	
